from collections.abc import AsyncIterator
from typing import Final, Optional

from typing_extensions import Unpack

from ncae_sdk._async.endpoint import AsyncEndpoint
from ncae_sdk._resource import ResourceId
from ncae_sdk._util import map_async
from ncae_sdk.resources._schema import Module, ModuleFilter, ModuleFilterModel


class AsyncModuleEndpoint(AsyncEndpoint):
    BASE_PATH: Final[str] = "admin/v1/module"

    async def list(self, **filters: Unpack[ModuleFilter]) -> AsyncIterator[Module]:
        results = self._list(self.BASE_PATH, ModuleFilterModel, filters)
        return map_async(Module.parse_api, results)

    async def find(self, **filters: Unpack[ModuleFilter]) -> Optional[Module]:
        result = await self._get_by_filters(self.BASE_PATH, ModuleFilterModel, filters)
        return Module.parse_api(result) if result else None

    async def get(self, rid: ResourceId) -> Optional[Module]:
        result = await self._get_by_id(self.BASE_PATH, rid)
        return Module.parse_api(result) if result else None
