from collections.abc import AsyncIterator
from typing import Final, Optional

from typing_extensions import Unpack

from ncae_sdk._async.endpoint import AsyncEndpoint
from ncae_sdk._resource import ResourceId
from ncae_sdk._util import map_async
from ncae_sdk.resources._schema import (
    BasePhase,
    BasePhaseFilter,
    BasePhaseFilterModel,
)


class AsyncBasePhaseEndpoint(AsyncEndpoint):
    BASE_PATH: Final[str] = "automation/v1/base-phase"

    async def list(self, **filters: Unpack[BasePhaseFilter]) -> AsyncIterator[BasePhase]:
        results = self._list(self.BASE_PATH, BasePhaseFilterModel, filters)
        return map_async(BasePhase.parse_api, results)

    async def find(self, **filters: Unpack[BasePhaseFilter]) -> Optional[BasePhase]:
        result = await self._get_by_filters(self.BASE_PATH, BasePhaseFilterModel, filters)
        return BasePhase.parse_api(result) if result else None

    async def get(self, rid: ResourceId) -> Optional[BasePhase]:
        result = await self._get_by_id(self.BASE_PATH, rid)
        return BasePhase.parse_api(result) if result else None
